
#include "VS_Shortcut_LEDS.h"
#define LED_DIR		DDRB
#define LED_PORT	PORTB
#define LED_BIT 	5

#define NB_ROWS 5
#define NB_COLS  7
#define MAX_KEYS 5


typedef struct {
    volatile unsigned char *ddr;
    volatile unsigned char *port;
    volatile unsigned char *pin;
    int n;
} IO_Config;

IO_Config cols[NB_COLS] = {
    {&DDRD, &PORTD, &PIND, 6},
    {&DDRD, &PORTD, &PIND, 5},
    {&DDRD, &PORTD, &PIND, 4},
    {&DDRD, &PORTD, &PIND, 3}
};

IO_Config rows[NB_ROWS] = {
    {&DDRB, &PORTB, NULL, 3},
    {&DDRB, &PORTB, NULL, 2},
    {&DDRB, &PORTB, NULL, 1},
    {&DDRB, &PORTB, NULL, 0}
};




typedef struct {
    uint8_t modifier1;
    uint8_t modifier2;
    uint8_t keyCode;
} KeyAction;



KeyAction keyActions[NB_ROWS][NB_COLS] = {
    // Ligne 1
    {
        {HID_KEYBOARD_MODIFIER_LEFTCTRL, 0, HID_KEYBOARD_SC_N},
        {HID_KEYBOARD_MODIFIER_LEFTCTRL, 0, HID_KEYBOARD_SC_O},
        {HID_KEYBOARD_MODIFIER_LEFTCTRL, 0, HID_KEYBOARD_SC_B},
		{HID_KEYBOARD_MODIFIER_LEFTCTRL, HID_KEYBOARD_MODIFIER_LEFTSHIFT, HID_KEYBOARD_SC_N}
        // Autres actions...
    },
    // Ligne 2
    {
        {HID_KEYBOARD_MODIFIER_LEFTCTRL, 0, HID_KEYBOARD_SC_Q},
        {HID_KEYBOARD_MODIFIER_LEFTCTRL, 0, HID_KEYBOARD_SC_X},
        {HID_KEYBOARD_MODIFIER_LEFTCTRL, 0, HID_KEYBOARD_SC_C},
		{HID_KEYBOARD_MODIFIER_LEFTCTRL, 0, HID_KEYBOARD_SC_V},
    }, 
	// Ligne 3
	   {
        {HID_KEYBOARD_MODIFIER_LEFTCTRL, 0, HID_KEYBOARD_SC_H},
        {HID_KEYBOARD_MODIFIER_LEFTCTRL, 0, HID_KEYBOARD_SC_F},
        {HID_KEYBOARD_MODIFIER_LEFTCTRL, 0, HID_KEYBOARD_SC_W},
		{HID_KEYBOARD_MODIFIER_LEFTCTRL, 0, HID_KEYBOARD_SC_Y},
    },
    {
        {HID_KEYBOARD_MODIFIER_LEFTCTRL, HID_KEYBOARD_MODIFIER_LEFTSHIFT, HID_KEYBOARD_SC_X},
        {HID_KEYBOARD_MODIFIER_LEFTCTRL, 0, HID_KEYBOARD_SC_F4},
        {0, 0, HID_KEYBOARD_SC_ESCAPE},
		{HID_KEYBOARD_MODIFIER_LEFTALT, 0, HID_KEYBOARD_SC_F4},
    },
    // Autres lignes...
};



static bool UsingReportProtocol = true;
static uint16_t IdleCount = 500;
static uint16_t IdleMSRemaining = 0;



int main(void)
{
	SetupHardware();

	LEDs_SetAllLEDs(LEDMASK_USB_NOTREADY);
	GlobalInterruptEnable();


	for (;;)
	{
		HID_Task();
		USB_USBTask();
	}


}

void SetupHardware(void)
{
#if (ARCH == ARCH_AVR8)
	/* Disable watchdog if enabled by bootloader/fuses */
	MCUSR &= ~(1 << WDRF);
	wdt_disable();

	/* Disable clock division */
	clock_prescale_set(clock_div_1);
#elif (ARCH == ARCH_XMEGA)
	/* Start the PLL to multiply the 2MHz RC oscillator to 32MHz and switch the CPU core to run from it */
	XMEGACLK_StartPLL(CLOCK_SRC_INT_RC2MHZ, 2000000, F_CPU);
	XMEGACLK_SetCPUClockSource(CLOCK_SRC_PLL);

	/* Start the 32MHz internal RC oscillator and start the DFLL to increase it to 48MHz using the USB SOF as a reference */
	XMEGACLK_StartInternalOscillator(CLOCK_SRC_INT_RC32MHZ);
	XMEGACLK_StartDFLL(CLOCK_SRC_INT_RC32MHZ, DFLL_REF_INT_USBSOF, F_USB);

	PMIC.CTRL = PMIC_LOLVLEN_bm | PMIC_MEDLVLEN_bm | PMIC_HILVLEN_bm;
#endif

	/* Hardware Initialization */
	Joystick_Init();
	LEDs_Init();
	USB_Init();
	Buttons_Init();
	CLKSEL0 = 0b00010101;   // sélection de l'horloge externe
	CLKSEL1 = 0b00001111;   // minimum de 8Mhz
	CLKPR = 0b10000000;     // modification du diviseur d'horloge (CLKPCE=1)
	CLKPR = 0;              // 0 pour pas de diviseur (diviseur de 1)



	//Allumer la 1ere ligne
	DDRD |= (1 << 6);
	PORTD |=(1 << 6);

	DDRB |= (1 << 3);
	DDRB |= (1 << 2);
	DDRB |= (1 << 1);
	DDRB |= (1 << 0);
	PORTB  &= ~(1 << 3);
	PORTB  &= ~(1 << 2);
	PORTB  &= ~(1 << 1);
	PORTB  &= ~(1 << 0);

	_delay_ms(1000);

	//Allumer la 2eme ligne
	DDRD &= ~ (1 << 6);
	PORTD &= ~(1 << 6);
	DDRD |= (1 << 5);
	PORTD |=(1 << 5);

	DDRB |= (1 << 3);
	DDRB |= (1 << 2);
	DDRB |= (1 << 1);
	DDRB |= (1 << 0);
	PORTB  &= ~(1 << 3);
	PORTB  &= ~(1 << 2);
	PORTB  &= ~(1 << 1);
	PORTB  &= ~(1 << 0);

	_delay_ms(1000);

	//Allumer la 3eme ligne
	DDRD &= ~ (1 << 5);
	PORTD &= ~(1 << 5);
	DDRD |= (1 << 4);
	PORTD |=(1 << 4);

	DDRB |= (1 << 3);
	DDRB |= (1 << 2);
	DDRB |= (1 << 1);
	DDRB |= (1 << 0);
	PORTB  &= ~(1 << 3);
	PORTB  &= ~(1 << 2);
	PORTB  &= ~(1 << 1);
	PORTB  &= ~(1 << 0);

	_delay_ms(1000);

	//Allumer la 4eme ligne
	DDRD &= ~ (1 << 4);
	PORTD &= ~(1 << 4);
	DDRD |= (1 << 3);
	PORTD |=(1 << 3);

	DDRB |= (1 << 3);
	DDRB |= (1 << 2);
	DDRB |= (1 << 1);
	DDRB |= (1 << 0);
	PORTB  &= ~(1 << 3);
	PORTB  &= ~(1 << 2);
	PORTB  &= ~(1 << 1);
	PORTB  &= ~(1 << 0);

	_delay_ms(1000);

	// Allumer les leds aux 4 coins
	DDRD &= ~ (1 << 3);
	PORTD &= ~(1 << 3);
	DDRB &= ~(1 << 1);
	DDRB &= ~(1 << 2);

	DDRD |= (1 << 6);
	PORTD |= (1 << 6);
	DDRB |= (1 << 0);
	DDRB |= (1 << 3);

	DDRD |= (1 << 3);
	PORTD |= (1 << 3);
	DDRB |= (1 << 0);
	DDRB |= (1 << 3);

	_delay_ms(1000);

	for (int col = 0; col < NB_COLS; col++) {
		*(cols[col].ddr) &= ~(1 << cols[col].n);
		*(cols[col].port) |= (1 << cols[col].n);
	}

	for (int row = 0; row < NB_ROWS; row++) {
		*(rows[row].ddr) |= (1 << rows[row].n);
		*(rows[row].port) &= ~(1 << rows[row].n);
	}


}


void EVENT_USB_Device_Connect(void)
{
	LEDs_SetAllLEDs(LEDMASK_USB_ENUMERATING);
	UsingReportProtocol = true;
}

void EVENT_USB_Device_Disconnect(void)
{
	LEDs_SetAllLEDs(LEDMASK_USB_NOTREADY);
}

void EVENT_USB_Device_ConfigurationChanged(void)
{
	bool ConfigSuccess = true;
	ConfigSuccess &= Endpoint_ConfigureEndpoint(KEYBOARD_IN_EPADDR, EP_TYPE_INTERRUPT, KEYBOARD_EPSIZE, 1);
	ConfigSuccess &= Endpoint_ConfigureEndpoint(KEYBOARD_OUT_EPADDR, EP_TYPE_INTERRUPT, KEYBOARD_EPSIZE, 1);
	USB_Device_EnableSOFEvents();
	LEDs_SetAllLEDs(ConfigSuccess ? LEDMASK_USB_READY : LEDMASK_USB_ERROR);
}

void EVENT_USB_Device_ControlRequest(void)
{
	switch (USB_ControlRequest.bRequest)
	{
		case HID_REQ_GetReport:
			if (USB_ControlRequest.bmRequestType == (REQDIR_DEVICETOHOST | REQTYPE_CLASS | REQREC_INTERFACE))
			{
				USB_KeyboardReport_Data_t KeyboardReportData;
				CreateKeyboardReport(&KeyboardReportData);

				Endpoint_ClearSETUP();
				Endpoint_Write_Control_Stream_LE(&KeyboardReportData, sizeof(KeyboardReportData));
				Endpoint_ClearOUT();
			}

			break;
		case HID_REQ_SetReport:
			if (USB_ControlRequest.bmRequestType == (REQDIR_HOSTTODEVICE | REQTYPE_CLASS | REQREC_INTERFACE))
			{
				Endpoint_ClearSETUP();

				while (!(Endpoint_IsOUTReceived()))
				{
					if (USB_DeviceState == DEVICE_STATE_Unattached)
					  return;
				}
				uint8_t LEDStatus = Endpoint_Read_8();

				Endpoint_ClearOUT();
				Endpoint_ClearStatusStage();
				ProcessLEDReport(LEDStatus);
			}

			break;
		case HID_REQ_GetProtocol:
			if (USB_ControlRequest.bmRequestType == (REQDIR_DEVICETOHOST | REQTYPE_CLASS | REQREC_INTERFACE))
			{
				Endpoint_ClearSETUP();
				Endpoint_Write_8(UsingReportProtocol);

				Endpoint_ClearIN();
				Endpoint_ClearStatusStage();
			}

			break;
		case HID_REQ_SetProtocol:
			if (USB_ControlRequest.bmRequestType == (REQDIR_HOSTTODEVICE | REQTYPE_CLASS | REQREC_INTERFACE))
			{
				Endpoint_ClearSETUP();
				Endpoint_ClearStatusStage();
				UsingReportProtocol = (USB_ControlRequest.wValue != 0);
			}

			break;
		case HID_REQ_SetIdle:
			if (USB_ControlRequest.bmRequestType == (REQDIR_HOSTTODEVICE | REQTYPE_CLASS | REQREC_INTERFACE))
			{
				Endpoint_ClearSETUP();
				Endpoint_ClearStatusStage();
				IdleCount = ((USB_ControlRequest.wValue & 0xFF00) >> 6);
			}

			break;
		case HID_REQ_GetIdle:
			if (USB_ControlRequest.bmRequestType == (REQDIR_DEVICETOHOST | REQTYPE_CLASS | REQREC_INTERFACE))
			{
				Endpoint_ClearSETUP();
				Endpoint_Write_8(IdleCount >> 2);

				Endpoint_ClearIN();
				Endpoint_ClearStatusStage();
			}

			break;
	}
}

void EVENT_USB_Device_StartOfFrame(void)
{	if (IdleMSRemaining)
	  IdleMSRemaining--;
}

void CreateKeyboardReport(USB_KeyboardReport_Data_t* const ReportData)
{

	uint8_t UsedKeyCodes      = 0;
	memset(ReportData, 0, sizeof(USB_KeyboardReport_Data_t));


	for (int row = 0; row < NB_ROWS; row++) {
		*(rows[row].port) &= ~(1 << rows[row].n);
		for (int col = 0; col < NB_COLS; col++) {
			unsigned char key = (*(cols[col].pin) & (1 << cols[col].n)) ? 0 : 1;
			if (key) {
				int touche = row * NB_COLS + col;
				if (touche < NB_ROWS * NB_COLS) {
					KeyAction action = keyActions[row][col];
					if (action.modifier1 != 0 && action.modifier2 != 0) {
						ReportData->Modifier = action.modifier1;
						ReportData->Modifier |= action.modifier2;
						ReportData->KeyCode[UsedKeyCodes++] = action.keyCode;
					} else if (action.modifier1 != 0) {
						ReportData->Modifier = action.modifier1;
						ReportData->KeyCode[UsedKeyCodes++] = action.keyCode;
					}else{
						ReportData->KeyCode[UsedKeyCodes++] = action.keyCode;
					}

				}
			}
		}
		*(rows[row].port) |= (1 << rows[row].n);
	}

}

void ProcessLEDReport(const uint8_t LEDReport)
{
	uint8_t LEDMask = LEDS_LED2;

	if (LEDReport & HID_KEYBOARD_LED_NUMLOCK)
	  LEDMask |= LEDS_LED1;

	if (LEDReport & HID_KEYBOARD_LED_CAPSLOCK)
	  LEDMask |= LEDS_LED3;

	if (LEDReport & HID_KEYBOARD_LED_SCROLLLOCK)
	  LEDMask |= LEDS_LED4;
	LEDs_SetAllLEDs(LEDMask);
}
void SendNextReport(void)
{
	static USB_KeyboardReport_Data_t PrevKeyboardReportData;
	USB_KeyboardReport_Data_t        KeyboardReportData;
	bool                             SendReport = false;

	CreateKeyboardReport(&KeyboardReportData);
	if (IdleCount && (!(IdleMSRemaining)))
	{
		IdleMSRemaining = IdleCount;
		SendReport = true;
	}
	else
	{
		SendReport = (memcmp(&PrevKeyboardReportData, &KeyboardReportData, sizeof(USB_KeyboardReport_Data_t)) != 0);
	}

	Endpoint_SelectEndpoint(KEYBOARD_IN_EPADDR);

	if (Endpoint_IsReadWriteAllowed() && SendReport)
	{
		PrevKeyboardReportData = KeyboardReportData;

		Endpoint_Write_Stream_LE(&KeyboardReportData, sizeof(KeyboardReportData), NULL);
		Endpoint_ClearIN();
	}
}

void ReceiveNextReport(void)
{
	Endpoint_SelectEndpoint(KEYBOARD_OUT_EPADDR);

	if (Endpoint_IsOUTReceived())
	{
		if (Endpoint_IsReadWriteAllowed())
		{
			uint8_t LEDReport = Endpoint_Read_8();

			ProcessLEDReport(LEDReport);
		}
		Endpoint_ClearOUT();
	}
}

void HID_Task(void)
{
	if (USB_DeviceState != DEVICE_STATE_Configured)
	  return;

	SendNextReport();

	ReceiveNextReport();
}

